--[[-------------------------------------------------------------------------
	QuickTargetMacros
    Copyright (C) 2010  Morsker of Blackhand-US
	Please contact Morsker through PM on forums.wowace.com.

	Aids the user in quickly making /target macros for immediate use.

	License:
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program. If not, see <http://www.gnu.org/licenses/>.

	Note:
		This AddOn's source code is specifically designed to work with
		World of Warcraft's interpreted AddOn system.
		You have an implicit license to use this AddOn with these facilities
		since that is its designated purpose as per:
		http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
---------------------------------------------------------------------------]]
QTMacros = LibStub("AceAddon-3.0"):NewAddon("QTMacros", "AceConsole-3.0", "AceEvent-3.0")
local QTMacros = QTMacros
local L = LibStub("AceLocale-3.0"):GetLocale("QTMacros", true)
local db
local mainButton
local keybindFrame

local QTM_DB_VERSION = 1 -- This is not the release version. It's for db versioning.


-- These generate OnKeyDown events, but aren't bindable and should be discarded.
local unbindableKeys = {
	LALT = 1,
	LCTRL = 1,
	LSHIFT = 1,
	RALT = 1,
	RCTRL = 1,
	RSHIFT = 1,
	UNKNOWN = 1,
}


-- rounds off to N significant decimal places.
-- e.g. 5.975, 2 -> 5.98
local function roundOff(val, nPlaces)
	coef = 1
	if nPlaces then
		coef = 10^nPlaces
	end
	return floor((val*coef) + 0.5) / coef
end


-- Create the main QT button, and assign it to "mainButton".
-- This is the button that holds the macrotext, and assigns a keybind to it when visible.
local function createMainButton()
	f = CreateFrame("Button", "QTMacrosMainButton", nil,
		"SecureHandlerShowHideTemplate,ActionButtonTemplate,SecureActionButtonTemplate")
	f.icon = _G[f:GetName().."Icon"]
	f.hotkey = _G[f:GetName().."HotKey"]
	f:SetFrameStrata("HIGH")
	f:SetMovable(true)
	f:SetClampedToScreen(true)
	f:Hide()

	if not db.buttonAnchor then
		db.buttonAnchor = { "CENTER", -100, -30 }
	end
	f:SetPoint(unpack(db.buttonAnchor))
	f:SetAttribute("type", "macro")
	if db.macroText then
		f:SetAttribute("macrotext", db.macroText)
	end
	f:SetAttribute("_onshow", [[
		local keybind = self:GetAttribute("keybind")
		if keybind then
			self:SetBindingClick(true, keybind, self:GetName(), "LeftButton")
		end
	]])
	f:SetAttribute("_onHide", [[
		self:ClearBindings()
	]])
	f:SetNormalTexture(nil)

	-- f:RegisterForClicks("LeftButtonDown", "RightButtonDown")
	f:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	f:RegisterForDrag("LeftButton")
	f.icon:SetTexture([[Interface\Addons\QuickTargetMacros\images\QT_icon.tga]])
	
	SecureHandlerWrapScript(f, "OnClick", f, [[
		if button == "RightButton" then
			self:Hide()
			return false
		end
	]])
	
	function f:ShowTooltip()
		if not InCombatLockdown() then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText("Quick Target Macros")
			GameTooltip:AddLine(self:GetAttribute("macrotext") or "")
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine(L["Click"], L["Target"])
			GameTooltip:AddDoubleLine(L["Right-Click"], L["Dismiss"])
			GameTooltip:AddDoubleLine(L["Alt-Click"], L["Move"])
			GameTooltip:Show();
		end
	end
	
	f:SetScript("OnEnter", function(self)
		self:ShowTooltip()
	end)
	
	f:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	
	f:SetScript("OnDragStart", function(self)
		if IsAltKeyDown() then
			GameTooltip:Hide()
			self:StartMoving()
		end
	end)
	
	f:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		self:SetUserPlaced(false)
		if self:GetNumPoints() > 0 then
			local point,_,_,x,y = self:GetPoint(1)
			db.buttonAnchor[1] = point
			db.buttonAnchor[2] = roundOff(x, 2)
			db.buttonAnchor[3] = roundOff(y, 2)
		end
		self:ShowTooltip()
	end)

	mainButton = f

	-- final stuff, after mainButton is assigned
	if db.keybind then
		QTMacros:SetKeybind(db.keybind)
	end
	if db.isShown then
		f:Show()
	end
end


-- Create the keybind frame and assign it to "keybindFrame". And Show it.
local function createAndShowKeybindFrame()
	local f = CreateFrame("Frame")
	f:SetFrameStrata("DIALOG")
	f:EnableKeyboard(true)
	f:SetClampedToScreen(true)
	f:Hide()
	f:SetPoint("CENTER", 0, UIParent:GetHeight() / 7)
	f:SetBackdrop({
		bgFile=[[Interface\Tooltips\UI-Tooltip-Background]],
		edgeFile=[[Interface\Tooltips\UI-Tooltip-Border]], tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	})
	f:SetBackdropColor(0, 0, 0)

	f.text = f:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	f.text:SetPoint("TOP", 0, -20)
	f.text:SetText(L[ [=[
Press a key to bind the /target macro.

Or press ESC to cancel.]=] ])

	f.text2 = f:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	f.text2:SetPoint("BOTTOM", 0, 20)
	f.text2:SetText(L[ [=[
(This is a temporary "override" that only lasts as long as you're
using the macro. It won't remove your previous binding.)]=] ])

	f:SetScript("OnShow", function(self)
		local width = max(self.text:GetWidth(), self.text2:GetWidth()) + 40
		local height = self.text:GetHeight() + self.text2:GetHeight() + 60
		self:SetWidth(width)
		self:SetHeight(height)
		
		self:RegisterEvent("PLAYER_REGEN_DISABLED") -- so we can hide in combat
	end)
	
	f:SetScript("OnHide", function(self)
		self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	end)
	
	f:SetScript("OnEvent", function(self, event, ...)
		if event == "PLAYER_REGEN_DISABLED" then
			QTMacros:Print(L["Combat! Aborting keybind selection."])
			self:Hide() -- hide in combat
		end
	end)

	f:SetScript("OnKeyDown", function(self, key)
		if unbindableKeys[key] then
			return
		end
		if key == "ESCAPE" then
			self:Hide()
			return
		end
		if GetBindingFromClick(key) == "SCREENSHOT" then
			TakeScreenshot()
			return
		end
		if IsShiftKeyDown() then
			key = "SHIFT-"..key
		end
		if IsControlKeyDown() then
			key = "CTRL-"..key
		end
		if IsAltKeyDown() then
			key = "ALT-"..key
		end

		QTMacros:SetKeybind(key)
		self:Hide()
		QTMacros:ActivateButton()
	end)
	
	keybindFrame = f
	keybindFrame:Show()
end


function QTMacros:SetKeybind(key)
	mainButton:SetAttribute("keybind", key)
	db.keybind = key
	if key then
		mainButton.hotkey:SetText(GetBindingText(key, "KEY_", true))
	else
		mainButton.hotkey:SetText(nil)
	end
end


function QTMacros:BuildMacro(macroText)
	mainButton:SetAttribute("macrotext", macroText)
	db.macroText = macroText
end


function QTMacros:ActivateButton()
	if mainButton:GetAttribute("keybind") then
		mainButton:Show()
	else
		keybindFrame:Show()
	end
end


function QTMacros:SlashQT(input)
	self:BuildMacro("/target " .. input .. "\n")
	self:ActivateButton()
end


function QTMacros:PLAYER_LOGOUT()
	-- db.isShown
	db.isShown = mainButton:IsShown()
	-- AddonLoader hack!
	if AddonLoader and AddonLoaderSV and db.isShown then
		-- We want AddonLoader to load us immediately if the button is shown, but we can't use our
		-- own vars for the setting because they don't exist at that point. So we stuff it in
		-- AddonLoader's vars. See the TOC for details.
		AddonLoaderSV.QTMacros_Hack_isShown = 1 -- Nil'd out in OnInitialize() the next time we load.
	end
end


function QTMacros:UpgradeDatabase()
	if db.db_version == nil then
		-- The first real version is 1. Old experimental versions had no db_version; we treat them as 0.
		db.db_version = 0
	end
	if db.db_version < 1 then
		-- This was for saving an actionID to drag a macro onto the bars in its place. Version 1 no longer uses it.
		db.savedActionID = nil
	end
	db.db_version = QTM_DB_VERSION
end


function QTMacros:OnInitialize()
	self:RegisterChatCommand("qt", "SlashQT")
	self:RegisterChatCommand("qtmobname", "SlashQT")
	self:RegisterChatCommand("qtname", "SlashQT")

	self:RegisterChatCommand("qtmshow", "ActivateButton")

	if AddonLoader and AddonLoaderSV then
		-- Cleanup any tricks we did with AddonLoader's saved vars.
		AddonLoaderSV.QTMacros_Hack_isShown = nil
	end

	QTMacrosPerCharDB = QTMacrosPerCharDB or {}
	db = QTMacrosPerCharDB
	if db.db_version ~= QTM_DB_VERSION then
		self:UpgradeDatabase()
	end

	createMainButton()
	keybindFrame = { Show = createAndShowKeybindFrame } -- Stub, standing in for Frame:Show()
end


function QTMacros:OnEnable()
	self:RegisterEvent("PLAYER_LOGOUT")
end


function QTMacros:OnDisable()
end
